# py_cyclo/controllers/complexity_controller.py
# pylint: disable=too-few-public-methods
import os
import sys
from typing import List

import click
from radon.visitors import Function

from py_cyclo.models.complexity_model import ComplexityModel
from py_cyclo.services.complexity_service import ComplexityService
from py_cyclo.views.complexity_view import ComplexityView


class ComplexityController:
    def __init__(
        self, service: ComplexityService, model: ComplexityModel, view: ComplexityView
    ) -> None:
        self.service = service
        self.model = model
        self.view = view

    def check_complexity(self) -> None:
        # Convert the path to an absolute path
        self.model.path = os.path.abspath(self.model.path)

        click.echo(f'Checking cyclomatic complexity in "{self.model.path}"...')

        files_to_analyze = self.service.get_files_to_analyze(
            self.model.path, self.model.exclude_dirs
        )
        if not files_to_analyze:
            click.echo("No Python files found to analyze.")
            sys.exit(0)

        results: List[Function] = self.service.analyze_files(files_to_analyze)
        if not results:
            click.echo("No output from radon.")
            sys.exit(1)

        exceeding = self.service.get_functions_exceeding_complexity(
            results, self.model.max_complexity
        )

        count = len(exceeding)
        if count > 0:
            click.echo(
                f"\n{count} functions exceed the maximum complexity "
                f"of {self.model.max_complexity}:"
            )
            self.view.display_complexity_table(exceeding)
        else:
            click.echo(
                f"There are no functions exceeding the maximum complexity "
                f"of {self.model.max_complexity}."
            )

        click.echo()

        within = self.service.get_functions_within_complexity(
            results, self.model.max_complexity
        )

        count = len(within)
        if count > 0:
            click.echo(
                f"\n{count} functions with a complexity < {self.model.max_complexity}:"
            )
            self.view.display_complexity_table(within)
        else:
            click.echo("There are no functions within the maximum complexity.")

        click.echo()

        max_score = self.service.get_max_score(results)
        click.echo(f"\nMaximum complexity: {max_score}\n")

        if max_score > self.model.max_complexity:
            click.echo(
                f"\nFAILED - Maximum complexity {self.model.max_complexity} "
                f"exceeded by {max_score}\n"
            )
            self.view.display_exceeded_complexity(results, self.model.max_complexity)
            sys.exit(1)

        self.view.display_max_score_not_exceeded(max_score)
        sys.exit(0)



# py_cyclo/services/complexity_service.py
import os
from typing import Any, List, Set

from radon.complexity import cc_visit
from radon.visitors import Function


class ComplexityService:
    def __init__(self):
        pass

    def get_files_to_analyze(self, path: str, exclude_dirs: Set[str]) -> List[str]:
        files_to_analyze = []
        for root, dirs, files in os.walk(path):
            dirs[:] = [d for d in dirs if d not in exclude_dirs]
            for file in files:
                if file.endswith(".py"):
                    files_to_analyze.append(os.path.join(root, file))

        return files_to_analyze

    def analyze_files(self, files_to_analyze: List[str]) -> List[Function]:
        results: List[Any] = []

        for file in files_to_analyze:
            with open(file, "r", encoding="utf-8") as f:
                code = f.read()
                file_results = cc_visit(code)
                for result in file_results:
                    result.filename = file
                results.extend(file_results)

        return results

    def get_max_score(self, results: List[Function]) -> int:
        max_score = 0
        for result in results:
            if isinstance(result, Function):
                max_score = max(max_score, result.complexity)
        return max_score

    def get_functions_exceeding_complexity(
        self, functions: List[Function], max_complexity: int
    ) -> List[Function]:
        return [func for func in functions if func.complexity > max_complexity]

    def get_functions_within_complexity(
        self, functions: List[Function], max_complexity: int
    ) -> List[Function]:
        return [func for func in functions if func.complexity <= max_complexity]



# python

import click

from py_cyclo.controllers.complexity_controller import ComplexityController
from py_cyclo.models.complexity_model import ComplexityModel
from py_cyclo.services.complexity_service import ComplexityService
from py_cyclo.views.complexity_view import ComplexityView


@click.command()
@click.argument("path", type=click.Path(exists=True, file_okay=False, dir_okay=True))
@click.option(
    "--max-complexity",
    "-m",
    default=12,
    help="The maximum allowed cyclomatic complexity.",
)
def check_complexity(path: str, max_complexity: int) -> None:
    """
    Check the cyclomatic complexity of the code.
    Fail if it exceeds the max_complexity.
    """
    main(path, max_complexity)


def main(path: str, max_complexity: int) -> None:
    """
    Main function to run the cyclomatic complexity check.
    """
    view = ComplexityView()
    service = ComplexityService()
    model = ComplexityModel(path, max_complexity, {"node_modules", ".venv", "venv"})
    controller = ComplexityController(service, model, view)

    controller.check_complexity()


if __name__ == "__main__":
    main(".", 12)



# py_cyclo/views/complexity_view.py
import os
import sys

import click
from tabulate import tabulate


class ComplexityView:
    def display_complexity_table(self, results) -> None:
        if not results:
            click.echo("No output from radon.")
            sys.exit(1)

        # Sort results by decreasing complexity
        sorted_results = sorted(results, key=lambda x: x.complexity, reverse=True)

        # Prepare data for tabulate
        table_data = [
            [
                result.name,
                result.complexity,
                result.lineno,
                os.path.relpath(result.filename),
            ]
            for result in sorted_results
        ]

        # Define table headers
        headers = ["Name", "Complexity", "Line No", "Filename"]

        # Print the table
        click.echo(tabulate(table_data, headers=headers, tablefmt="grid"))

    def display_radon_results(self, results) -> None:
        if not results:
            click.echo("No output from radon.")
            sys.exit(1)

        click.echo("\nCyclomatic Complexity Results:")
        click.echo("-------------------------------")
        for result in results:
            click.echo(f"{result.name}: {result.complexity}")

    def display_exceeded_complexity(self, results, max_complexity) -> None:
        if not results:
            click.echo("No output from radon.")
            sys.exit(1)

        click.echo("\nFunctions with complexity greater than the maximum allowed:")
        for result in results:
            if result.complexity > max_complexity:
                click.echo(f"{result.name}: {result.complexity}")

    def display_max_score_not_exceeded(self, max_score) -> None:
        click.echo(f"\nMaximum complexity not exceeded: {max_score}\n")



# py_cyclo/models/complexity_model.py
from typing import Optional, Set


class ComplexityModel:
    def __init__(
        self,
        path: Optional[str] = None,
        max_complexity: Optional[int] = 0,
        exclude_dirs: Optional[Set[str]] = None,
    ):
        self.path = path
        self.max_complexity = max_complexity
        self.exclude_dirs = exclude_dirs

    def __str__(self):
        return (
            f"ComplexityModel(path={self.path}, max_complexity={self.max_complexity}, "
            f"exclude_dirs={sorted(list(self.exclude_dirs))})"
        )

    def __repr__(self):
        return self.__str__()
